import { privateProperty } from "@default-js/defaultjs-common-utils/src/PrivateProperty";
import { toNodeName, define } from "@default-js/defaultjs-html-components/src/utils/DefineComponentHelper";
import { componentEventname } from "@default-js/defaultjs-html-components/src/utils/EventHelper";
import { Renderer, Template } from "@default-js/defaultjs-template-language";
import { loadTemplate, ATTR_TEMPLATE } from "@default-js/defaultjs-html-components/src/utils/TemplateHelper";
import Component from "@default-js/defaultjs-html-components/src/Component";

const NODENAME = toNodeName("pagination");

const TEMPLATE = Template.load(
	`
<nav class="pagination" jstl-if="\${pages.length > 1}">
	<ul>
		<li class="pagination-first \${page == 1 ? 'disabled' : ''}" ?data-page="\${page != 1}" data-page="1">first</li>
		<li class="pagination-back \${page == 1 ? 'disabled' : ''}" ?data-page="\${page != 1}" data-page="\${page - 1}">prev</li>

		<jstl jstl-foreach="\${pages}">
			<li class="pagination-page \${page == item ? 'active' : ''}" ?data-page="\${page != item}" data-page="\${item}">\${item}</li>
		</jstl>

		<li class="pagination-next \${page >= count ? 'disabled' : ''}" ?data-page="\${count > page}" data-page="\${page + 1}">next</li>
		<li class="pagination-last \${page >= count ? 'disabled' : ''}" ?data-page="\${count > page}" data-page="\${count}">last</li>
	</ul>
</nav>
`,
	false,
);

const toData = (page, count, size) => {
	const pages = [];
	const meridian = Math.floor(size / 2);
	let start = 1;
	let end = size;
	if (size > count) end = count;
	else if (page + meridian > count) {
		end = count;
		start = end - size + 1;
	} else if (page - meridian > 1) {
		end = page + meridian;
		start = end - size + 1;
	}
	for (let i = start; i <= end; i++) pages.push(i);

	return { start, end, pages, page, count, size };
};

const ATTR_PAGE = "page";
const ATTR_COUNT = "count";
const ATTR_SIZE = "size";
const ATTR_DISABLED_SHADOW_DOM = "disabled-shadow-dom";
const ATTR_DATA_PAGE = "data-page";
const ATTRIBUTES = [ATTR_PAGE, ATTR_COUNT, ATTR_SIZE, ATTR_TEMPLATE];

const EVENT__TO_PAGE = componentEventname("to-page", NODENAME);
const EVENT__CHANGE = componentEventname("change", NODENAME);

const PRIVATE__RENDERER = "renderer";

class HTMLPaginationElement extends Component {
	static get observedAttributes() {
		return ATTRIBUTES;
	}

	static get NODENAME() {
		return NODENAME;
	}

	constructor() {
		super();
	}

	async init() {
		await super.init();
		if (!this.ready.resolved) {
			const defaultTemplate = await TEMPLATE;
			const template = await loadTemplate(this, defaultTemplate);
			if (!this.disabledShadowDom && template != defaultTemplate)
				this.attachShadow({ mode: "open" });
			this.root.on("click", (event) => {
				const { target } = event;
				let page = Number.NaN;
				page = parseInt(target.attr(ATTR_DATA_PAGE));
				if (Number.isNaN(page)) {
					const parent = target.parent("[" + ATTR_DATA_PAGE + "]");
					if (parent) page = parseInt(parent.attr(ATTR_DATA_PAGE));
				}
				if (!Number.isNaN(page)) {
					this.trigger(EVENT__TO_PAGE, page);
					event.preventDefault();
					event.stopPropagation();
				}
			});

			privateProperty(this, PRIVATE__RENDERER, new Renderer({ template }));

			this.on(EVENT__CHANGE, () => {
				this.render();
			});
		}
		this.render();
	}

	get page() {
		return parseInt(this.attr(ATTR_PAGE) || "1");
	}
	set page(page) {
		this.attr(ATTR_PAGE, page);
	}

	get count() {
		return parseInt(this.attr(ATTR_COUNT) || "0");
	}
	set count(count) {
		this.attr(ATTR_COUNT, count);
	}

	get size() {
		return parseInt(this.attr(ATTR_SIZE) || "10");
	}
	set size(size) {
		this.attr(ATTR_SIZE, size);
	}

	get disabledShadowDom() {
		return this.hasAttribute(ATTR_DISABLED_SHADOW_DOM);
	}

	async render() {
		const renderer = privateProperty(this, PRIVATE__RENDERER);
		renderer.render({ data: toData(this.page, this.count, this.size), container: this.root });
	}
}

define(HTMLPaginationElement);
export default HTMLPaginationElement;
